@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2010 The PHP Group                                |
+  | Copyright (c) 1997-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 */
  
-/* $Id: bz2.c 305507 2010-11-18 15:22:22Z pajoye $ */
+/* $Id: bz2.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -107,7 +107,7 @@ static const zend_function_entry bz2_functions[] = {
 	PHP_FE(bzerror,      arginfo_bzerror)
 	PHP_FE(bzcompress,   arginfo_bzcompress)
 	PHP_FE(bzdecompress, arginfo_bzdecompress)
-	{NULL, NULL, NULL}
+	PHP_FE_END
 };
 
 zend_module_entry bz2_module_entry = {
