@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: com_extension.c 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id: com_extension.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -228,7 +228,7 @@ const zend_function_entry com_dotnet_functions[] = {
 	PHP_FE(com_message_pump, arginfo_com_message_pump)
 	PHP_FE(com_load_typelib, arginfo_com_load_typelib)
 	PHP_FE(com_get_active_object, arginfo_com_get_active_object)
-	{ NULL, NULL, NULL }
+	PHP_FE_END
 };
 
 /* {{{ com_dotnet_module_entry
