@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: com_persist.c 305507 2010-11-18 15:22:22Z pajoye $ */
+/* $Id: com_persist.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 /* Infrastructure for working with persistent COM objects.
  * Implements: IStream* wrapper for PHP streams.
@@ -697,7 +697,7 @@ static const zend_function_entry com_persist_helper_methods[] = {
 	CPH_ME(InitNew, NULL)
 	CPH_ME(LoadFromStream, NULL)
 	CPH_ME(SaveToStream, NULL)
-	{NULL, NULL, NULL}
+	PHP_FE_END
 };
 
 static void helper_free_storage(void *obj TSRMLS_DC)
