@@ -16,15 +16,17 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: interval.c 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id: interval.c 320563 2011-12-07 08:39:43Z pajoye $ */
 
 #include "timelib.h"
+#include <math.h>
 
 timelib_rel_time *timelib_diff(timelib_time *one, timelib_time *two)
 {
 	timelib_rel_time *rt;
 	timelib_time *swp;
 	timelib_sll dst_h_corr = 0, dst_m_corr = 0;
+	timelib_time one_backup, two_backup;
 
 	rt = timelib_rel_time_ctor();
 	rt->invert = 0;
@@ -45,6 +47,10 @@ timelib_rel_time *timelib_diff(timelib_time *one, timelib_time *two)
 		dst_m_corr = ((two->z - one->z) % 3600) / 60;
 	}
 
+	/* Save old TZ info */
+	memcpy(&one_backup, one, sizeof(one_backup));
+	memcpy(&two_backup, two, sizeof(two_backup));
+
     timelib_apply_localtime(one, 0);
     timelib_apply_localtime(two, 0);
 
@@ -56,10 +62,11 @@ timelib_rel_time *timelib_diff(timelib_time *one, timelib_time *two)
 	rt->s = two->s - one->s;
 	rt->days = abs(floor((one->sse - two->sse - (dst_h_corr * 3600) - (dst_m_corr * 60)) / 86400));
 
-	timelib_do_rel_normalize(one, rt);
+	timelib_do_rel_normalize(rt->invert ? one : two, rt);
 
-    timelib_apply_localtime(one, 1);
-    timelib_apply_localtime(two, 1);
+	/* Restore old TZ info */
+	memcpy(one, &one_backup, sizeof(one_backup));
+	memcpy(two, &two_backup, sizeof(two_backup));
 
 	return rt;
 }
