@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2006 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: dba_db4.c,v 1.15.2.3 2006/02/18 17:11:06 helly Exp $ */
+/* $Id: dba_db4.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -36,10 +36,27 @@
 #include <db.h>
 #endif
 
-static void php_dba_db4_errcall_fcn(const char *errpfx, char *msg)
+static void php_dba_db4_errcall_fcn(
+#if (DB_VERSION_MAJOR > 4 || (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3))
+	const DB_ENV *dbenv, 
+#endif
+	const char *errpfx, const char *msg)
 {
 	TSRMLS_FETCH();
-	
+
+#if (DB_VERSION_MAJOR == 5 || (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR == 8))
+/* Bug 51086, Berkeley DB 4.8.26 */
+/* This code suppresses a BDB 4.8+ error message, thus keeping PHP test compatibility */
+	{
+		char *function = get_active_function_name(TSRMLS_C);
+		if (function && (!strcmp(function,"dba_popen") || !strcmp(function,"dba_open"))
+			&& (!strncmp(msg, "fop_read_meta", sizeof("fop_read_meta")-1)
+				|| !strncmp(msg, "BDB0004 fop_read_meta", sizeof("BDB0004 fop_read_meta")-1))) {
+			return;
+		}
+	}
+#endif
+
 	php_error_docref(NULL TSRMLS_CC, E_NOTICE, "%s%s", errpfx?errpfx:"", msg);
 }
 
@@ -63,6 +80,7 @@ DBA_OPEN_FUNC(db4)
 	struct stat check_stat;
 	int s = VCWD_STAT(info->path, &check_stat);
 
+#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR <= 7)  /* Bug 51086 */
 	if (!s && !check_stat.st_size) {
 		info->mode = DBA_TRUNC; /* force truncate */
 	}
@@ -76,12 +94,25 @@ DBA_OPEN_FUNC(db4)
 		(info->mode == DBA_CREAT && !s) ? 0 :
 		info->mode == DBA_WRITER ? 0         : 
 		info->mode == DBA_TRUNC ? DB_CREATE | DB_TRUNCATE : -1;
+#else
+	if (!s && !check_stat.st_size) {
+		info->mode = DBA_CREAT; /* force creation */
+	}
+
+	type = info->mode == DBA_READER ? DB_UNKNOWN :
+		(info->mode == DBA_TRUNC || info->mode == DBA_CREAT) ? DB_BTREE :
+		s ? DB_BTREE : DB_UNKNOWN;
+	  
+	gmode = info->mode == DBA_READER ? DB_RDONLY :
+		info->mode == DBA_CREAT ? DB_CREATE : 
+		info->mode == DBA_WRITER ? 0         : 
+		info->mode == DBA_TRUNC ? DB_CREATE | DB_TRUNCATE : -1;
+#endif
 
 	if (gmode == -1) {
 		return FAILURE; /* not possible */
 	}
 
-	gmode |= DB_INIT_LOCK;
 	if (info->flags & DBA_PERSISTENT) {
 		gmode |= DB_THREAD;
 	}
@@ -91,14 +122,10 @@ DBA_OPEN_FUNC(db4)
 		filemode = Z_LVAL_PP(info->argv[0]);
 	}
 
-#ifdef DB_FCNTL_LOCKING
-	gmode |= DB_FCNTL_LOCKING;
-#endif
-
 	if ((err=db_create(&dbp, NULL, 0)) == 0) {
 	    dbp->set_errcall(dbp, php_dba_db4_errcall_fcn);
 	    if (
-#if (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 1)
+#if (DB_VERSION_MAJOR > 4 || (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 1))
 			(err=dbp->open(dbp, 0, info->path, NULL, type, gmode, filemode)) == 0) {
 #else
 			(err=dbp->open(dbp, info->path, NULL, type, gmode, filemode)) == 0) {
@@ -176,7 +203,15 @@ DBA_EXISTS_FUNC(db4)
 	DB4_GKEY;
 	
 	memset(&gval, 0, sizeof(gval));
+	
+	if (info->flags & DBA_PERSISTENT) {
+		gval.flags |= DB_DBT_MALLOC;
+	}
+
 	if (!dba->dbp->get(dba->dbp, NULL, &gkey, &gval, 0)) {
+		if (info->flags & DBA_PERSISTENT) {
+			free(gval.data);
+		}
 		return SUCCESS;
 	}
 	return FAILURE;
@@ -220,7 +255,7 @@ DBA_NEXTKEY_FUNC(db4)
 		gkey.flags |= DB_DBT_MALLOC;
 		gval.flags |= DB_DBT_MALLOC;
 	}
-	if (dba->cursor->c_get(dba->cursor, &gkey, &gval, DB_NEXT) == 0) {
+	if (dba->cursor && dba->cursor->c_get(dba->cursor, &gkey, &gval, DB_NEXT) == 0) {
 		if (gkey.data) {
 			nkey = estrndup(gkey.data, gkey.size);
 			if (newlen) *newlen = gkey.size;
