@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2010 The PHP Group                                |
+  | Copyright (c) 1997-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.0 of the PHP license,       |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
   |         Ilia Alshanetsky <ilia@prohost.org>                          |
   +----------------------------------------------------------------------+
 
-  $Id: enchant.c 305507 2010-11-18 15:22:22Z pajoye $
+  $Id: enchant.c 321634 2012-01-01 13:15:04Z felipe $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -146,8 +146,7 @@ function_entry enchant_functions[] = {
 	PHP_FE(enchant_dict_get_error, 			arginfo_enchant_broker_free_dict)
 	PHP_FE(enchant_dict_describe, 			arginfo_enchant_broker_free_dict)
 	PHP_FE(enchant_dict_quick_check, 		arginfo_enchant_dict_quick_check)
-
-	{NULL, NULL, NULL}	/* Must be the last line in enchant_functions[] */
+	PHP_FE_END
 };
 /* }}} */
 
@@ -327,7 +326,7 @@ PHP_MINFO_FUNCTION(enchant)
 #elif defined(HAVE_ENCHANT_BROKER_SET_PARAM)
 	php_info_print_table_row(2, "Libenchant Version", "1.5.0 or later");
 #endif
-	php_info_print_table_row(2, "Revision", "$Revision: 305507 $");
+	php_info_print_table_row(2, "Revision", "$Revision: 321634 $");
 	php_info_print_table_end();
 
 	php_info_print_table_start();
