@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2006 The PHP Group                                |
+  | Copyright (c) 1997-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: hash_whirlpool.c,v 1.3.2.3 2006/01/01 12:50:07 sniper Exp $ */
+/* $Id: hash_whirlpool.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #include "php_hash.h"
 
@@ -433,10 +433,11 @@ PHP_HASH_API void PHP_WHIRLPOOLFinal(unsigned char digest[64], PHP_WHIRLPOOL_CTX
     memset(context, 0, sizeof(*context));
 }
 
-php_hash_ops php_hash_whirlpool_ops = {
+const php_hash_ops php_hash_whirlpool_ops = {
 	(php_hash_init_func_t) PHP_WHIRLPOOLInit,
 	(php_hash_update_func_t) PHP_WHIRLPOOLUpdate,
 	(php_hash_final_func_t) PHP_WHIRLPOOLFinal,
+	(php_hash_copy_func_t) php_hash_copy,
 	64,
 	64,
 	sizeof(PHP_WHIRLPOOL_CTX)
