@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2007 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: ibase_query.c,v 1.23.2.1.2.5 2007/01/22 09:08:28 tony2001 Exp $ */
+/* $Id: ibase_query.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -49,6 +49,7 @@ typedef struct {
 typedef struct {
 	ibase_db_link *link;
 	ibase_trans *trans;
+	struct _ib_query *query;
 	isc_stmt_handle stmt;
 	unsigned short type;
 	unsigned char has_more_rows, statement_type;
@@ -56,9 +57,10 @@ typedef struct {
 	ibase_array out_array[1]; /* last member */
 } ibase_result;
 
-typedef struct {
+typedef struct _ib_query {
 	ibase_db_link *link;
 	ibase_trans *trans;
+	ibase_result *result;
 	int result_res_id;
 	isc_stmt_handle stmt;
 	XSQLDA *in_sqlda, *out_sqlda;
@@ -116,6 +118,24 @@ static void _php_ibase_free_xsqlda(XSQLDA *sqlda) /* {{{ */
 }
 /* }}} */
 
+static void _php_ibase_free_stmt_handle(ibase_db_link *link, isc_stmt_handle stmt TSRMLS_DC) /* {{{ */
+{
+	static char info[] = { isc_info_base_level, isc_info_end };
+
+	if (stmt) {
+		char res_buf[8];
+		IBDEBUG("Dropping statement handle (free_stmt_handle)...");
+		/* Only free statement if db-connection is still open */
+		if (SUCCESS == isc_database_info(IB_STATUS, &link->handle, 
+							sizeof(info), info, sizeof(res_buf), res_buf)) {
+			if (isc_dsql_free_statement(IB_STATUS, &stmt, DSQL_drop)) {
+				_php_ibase_error(TSRMLS_C);
+			}
+		}
+	}
+}
+/* }}} */
+
 static void _php_ibase_free_result(zend_rsrc_list_entry *rsrc TSRMLS_DC) /* {{{ */
 {
 	ibase_result *ib_result = (ibase_result *) rsrc->ptr;
@@ -123,9 +143,11 @@ static void _php_ibase_free_result(zend_rsrc_list_entry *rsrc TSRMLS_DC) /* {{{
 	IBDEBUG("Freeing result by dtor...");
 	if (ib_result) {
 		_php_ibase_free_xsqlda(ib_result->out_sqlda);
-		if (ib_result->stmt && ib_result->type != EXECUTE_RESULT) {
-			IBDEBUG("Dropping statement handle (free_result dtor)...");
-			isc_dsql_free_statement(IB_STATUS, &ib_result->stmt, DSQL_drop);
+		if (ib_result->query != NULL) {
+			IBDEBUG("query still valid; don't drop statement handle");
+			ib_result->query->result = NULL;	/* Indicate to query, that result is released */
+		} else {
+			_php_ibase_free_stmt_handle(ib_result->link, ib_result->stmt TSRMLS_CC);
 		}
 		efree(ib_result);
 	}
@@ -142,11 +164,11 @@ static void _php_ibase_free_query(ibase_query *ib_query TSRMLS_DC) /* {{{ */
 	if (ib_query->out_sqlda) {
 		efree(ib_query->out_sqlda);
 	}
-	if (ib_query->stmt) {
-		IBDEBUG("Dropping statement handle (free_query)...");
-		if (isc_dsql_free_statement(IB_STATUS, &ib_query->stmt, DSQL_drop)) {
-			_php_ibase_error(TSRMLS_C);
-		}
+	if (ib_query->result != NULL) {
+		IBDEBUG("result still valid; don't drop statement handle");
+		ib_query->result->query = NULL;	/* Indicate to result, that query is released */
+	} else {
+		_php_ibase_free_stmt_handle(ib_query->link, ib_query->stmt TSRMLS_CC);
 	}
 	if (ib_query->in_array) {
 		efree(ib_query->in_array);
@@ -166,11 +188,7 @@ static void php_ibase_free_query_rsrc(zend_rsrc_list_entry *rsrc TSRMLS_DC) /* {
 
 	if (ib_query != NULL) {
 		IBDEBUG("Preparing to free query by dtor...");
-
 		_php_ibase_free_query(ib_query TSRMLS_CC);
-		if (ib_query->statement_type != isc_info_sql_stmt_exec_procedure) {
-			zend_list_delete(ib_query->result_res_id);
-		}
 		efree(ib_query);
 	}
 }
@@ -302,9 +320,16 @@ static int _php_ibase_alloc_query(ibase_query *ib_query, ibase_db_link *link, /*
 	static char info_type[] = {isc_info_sql_stmt_type};
 	char result[8];
 
+	/* Return FAILURE, if querystring is empty */
+	if (*query == '\0') {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Querystring empty.");
+		return FAILURE;
+	}
+
 	ib_query->link = link;
 	ib_query->trans = trans;
 	ib_query->result_res_id = 0;
+	ib_query->result = NULL;
 	ib_query->stmt = NULL;
 	ib_query->in_array = NULL;
 	ib_query->out_array = NULL;
@@ -603,9 +628,8 @@ static int _php_ibase_bind_array(zval *val, char *buf, unsigned long buf_size, /
 					break;
 				default:
 					convert_to_string(val);
-					strncpy(buf, Z_STRVAL_P(val), array->el_size);
-					buf[array->el_size-1] = '\0';
-			}	
+					strlcpy(buf, Z_STRVAL_P(val), buf_size);
+			}
 		}
 	}
 	return SUCCESS;
@@ -648,14 +672,7 @@ static int _php_ibase_bind(XSQLDA *sqlda, zval ***b_vars, BIND_BUF *buf, /* {{{
 				if (! force_null) break;
 
 			case IS_NULL:
-
-				/* complain if this field doesn't allow NULL values */
-				if (! (var->sqltype & 1)) {
-					_php_ibase_module_error("Parameter %d: non-empty value required" TSRMLS_CC, i+1);
-					rv = FAILURE;
-				} else {
 					buf[i].sqlind = -1;
-				}
 
 				if (var->sqltype & SQL_ARRAY) ++array_cnt;
 
@@ -669,13 +686,17 @@ static int _php_ibase_bind(XSQLDA *sqlda, zval ***b_vars, BIND_BUF *buf, /* {{{
 		var->sqldata = (void*)&buf[i].val;
 
 		switch (var->sqltype & ~1) {
-			struct tm *t, tmbuf;
+			struct tm t;
 
 			case SQL_TIMESTAMP:
 			case SQL_TYPE_DATE:
 			case SQL_TYPE_TIME:
 				if (Z_TYPE_P(b_var) == IS_LONG) {
-					t = php_gmtime_r(&Z_LVAL_P(b_var), &tmbuf);
+					struct tm *res;
+					res = php_gmtime_r(&Z_LVAL_P(b_var), &t);
+					if (!res) {
+						return FAILURE;
+					}
 				} else {
 #ifdef HAVE_STRPTIME
 					char *format = INI_STR("ibase.timestampformat");
@@ -689,7 +710,7 @@ static int _php_ibase_bind(XSQLDA *sqlda, zval ***b_vars, BIND_BUF *buf, /* {{{
 						case SQL_TYPE_TIME:
 							format = INI_STR("ibase.timeformat");
 					}
-					if (!strptime(Z_STRVAL_P(b_var), format, t)) {
+					if (!strptime(Z_STRVAL_P(b_var), format, &t)) {
 						/* strptime() cannot handle it, so let IB have a try */
 						break;
 					}
@@ -700,13 +721,13 @@ static int _php_ibase_bind(XSQLDA *sqlda, zval ***b_vars, BIND_BUF *buf, /* {{{
 
 				switch (var->sqltype & ~1) {
 					default: /* == case SQL_TIMESTAMP */
-						isc_encode_timestamp(t, &buf[i].val.tsval);
+						isc_encode_timestamp(&t, &buf[i].val.tsval);
 						break;
 					case SQL_TYPE_DATE:
-						isc_encode_sql_date(t, &buf[i].val.dtval);
+						isc_encode_sql_date(&t, &buf[i].val.dtval);
 						break;
 					case SQL_TYPE_TIME:
-						isc_encode_sql_time(t, &buf[i].val.tmval);
+						isc_encode_sql_time(&t, &buf[i].val.tmval);
 						break;
 				}
 				continue;
@@ -926,7 +947,10 @@ static int _php_ibase_exec(INTERNAL_FUNCTION_PARAMETERS, ibase_result **ib_resul
 		res = emalloc(sizeof(ibase_result)+sizeof(ibase_array)*max(0,ib_query->out_array_cnt-1));
 		res->link = ib_query->link;
 		res->trans = ib_query->trans;
-		res->stmt = ib_query->stmt; 
+		res->stmt = ib_query->stmt;
+		/* ib_result and ib_query point at each other to handle release of statement handle properly */
+		res->query = ib_query;
+		ib_query->result = res;
 		res->statement_type = ib_query->statement_type;
 		res->has_more_rows = 1;
 
@@ -1000,8 +1024,7 @@ static int _php_ibase_exec(INTERNAL_FUNCTION_PARAMETERS, ibase_result **ib_resul
 				if (affected_rows) {
 					RETVAL_LONG(affected_rows);
 				} else {
-					/* this return value evaluates to bool(true) and to int(0) */
-					RETVAL_STRINGL("0 ",2,1);
+					RETVAL_TRUE;
 				}
 				break;
 			}
@@ -1039,7 +1062,7 @@ PHP_FUNCTION(ibase_query)
 {
 	zval *zlink, *ztrans, ***bind_args = NULL;
 	char *query;
-	int bind_i, query_len;
+	int bind_i, query_len, bind_num;
 	long trans_res_id = 0;
 	ibase_db_link *ib_link = NULL;
 	ibase_trans *trans = NULL;
@@ -1145,11 +1168,9 @@ PHP_FUNCTION(ibase_query)
 			if (bind_n < expected_n) {
 				break;
 			}
-		} else if (bind_n > 0) {
-			bind_args = (zval ***) safe_emalloc(sizeof(zval **), ZEND_NUM_ARGS(), 0);
-
-			if (FAILURE == zend_get_parameters_array_ex(ZEND_NUM_ARGS(), bind_args)) {
-				break;
+		} else if (bind_n > 0) {		
+			if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "+", &bind_args, &bind_num) == FAILURE) {
+				return;
 			}
 		}
 	
@@ -1182,42 +1203,32 @@ PHP_FUNCTION(ibase_query)
 PHP_FUNCTION(ibase_affected_rows)
 {
 	ibase_trans *trans = NULL;
-
+	ibase_db_link *ib_link;
+	zval *arg = NULL;
+		
 	RESET_ERRMSG;
+	
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|r", &arg) == FAILURE) {
+		return;
+	}
 
-	switch (ZEND_NUM_ARGS()) {
+	if (!arg) {
+		ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, NULL, IBG(default_link), LE_LINK, le_link, le_plink);
+		if (ib_link->tr_list == NULL || ib_link->tr_list->trans == NULL) {
+			RETURN_FALSE;
+		}
+		trans = ib_link->tr_list->trans;
+	} else {
+		/* one id was passed, could be db or trans id */
+		_php_ibase_get_link_trans(INTERNAL_FUNCTION_PARAM_PASSTHRU, &arg, &ib_link, &trans);
+		if (trans == NULL) {
+			ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, &arg, -1, LE_LINK, le_link, le_plink);
 
-		ibase_db_link *ib_link;
-		zval **arg;
-
-		case 0:
-			ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, NULL, IBG(default_link), LE_LINK,
-				le_link, le_plink);
 			if (ib_link->tr_list == NULL || ib_link->tr_list->trans == NULL) {
 				RETURN_FALSE;
 			}
 			trans = ib_link->tr_list->trans;
-			break;
-
-		case 1: 
-			if (zend_get_parameters_ex(1, &arg) == FAILURE) {
-				RETURN_FALSE;
-			}
-			/* one id was passed, could be db or trans id */
-			_php_ibase_get_link_trans(INTERNAL_FUNCTION_PARAM_PASSTHRU, arg, &ib_link, &trans);
-			if (trans == NULL) {			
-				ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, arg, -1, LE_LINK, le_link, le_plink);
-
-				if (ib_link->tr_list == NULL || ib_link->tr_list->trans == NULL) {
-					RETURN_FALSE;
-				}
-				trans = ib_link->tr_list->trans;
-			}
-			break;
-
-		default:
-			WRONG_PARAM_COUNT;
-			break;
+		}
 	}
 	RETURN_LONG(trans->affected_rows);
 }
@@ -1248,21 +1259,20 @@ PHP_FUNCTION(ibase_num_rows)
 	 *   sets.
 	 */
 
-	zval **result_arg;
+	zval *result_arg;
 	ibase_result *ib_result;
 	static char info_count[] = {isc_info_sql_records};
 	char result[64];
 
 	RESET_ERRMSG;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &result_arg) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &result_arg) == FAILURE) {
+		return;
 	}
 
-	ZEND_FETCH_RESOURCE(ib_result, ibase_result *, result_arg, -1, LE_RESULT, le_result);
+	ZEND_FETCH_RESOURCE(ib_result, ibase_result *, &result_arg, -1, LE_RESULT, le_result);
 
-	if (isc_dsql_sql_info(IB_STATUS, &ib_result->stmt, sizeof(info_count), info_count, 
-			sizeof(result), result)) {
+	if (isc_dsql_sql_info(IB_STATUS, &ib_result->stmt, sizeof(info_count), info_count, sizeof(result), result)) {
 		_php_ibase_error(TSRMLS_C);
 		RETURN_FALSE;
 	}
@@ -1285,9 +1295,23 @@ PHP_FUNCTION(ibase_num_rows)
 static int _php_ibase_var_zval(zval *val, void *data, int type, int len, /* {{{ */
 	int scale, int flag TSRMLS_DC)
 {
-	static ISC_INT64 const scales[] = { 1, 10, 100, 1000, 10000, 100000, 1000000, 100000000, 1000000000, 
-		1000000000, LL_LIT(10000000000),LL_LIT(100000000000),LL_LIT(10000000000000),LL_LIT(100000000000000),
-		LL_LIT(1000000000000000),LL_LIT(1000000000000000),LL_LIT(1000000000000000000) };
+	static ISC_INT64 const scales[] = { 1, 10, 100, 1000, 
+		10000, 
+		100000, 
+		1000000, 
+		10000000,
+		100000000, 
+		1000000000, 
+		LL_LIT(10000000000), 
+		LL_LIT(100000000000),
+		LL_LIT(1000000000000), 
+		LL_LIT(10000000000000), 
+		LL_LIT(100000000000000),
+		LL_LIT(1000000000000000),
+		LL_LIT(10000000000000000), 
+		LL_LIT(100000000000000000), 
+		LL_LIT(1000000000000000000)
+	};
 
 	switch (type & ~1) {
 		unsigned short l;
@@ -1317,17 +1341,17 @@ static int _php_ibase_var_zval(zval *val, void *data, int type, int len, /* {{{
 			goto _sql_long;
 #else
 			if (scale == 0) {
-				l = sprintf(string_data, "%" LL_MASK "d", *(ISC_INT64 *) data);
+				l = slprintf(string_data, sizeof(string_data), "%" LL_MASK "d", *(ISC_INT64 *) data);
 				ZVAL_STRINGL(val,string_data,l,1);
 			} else {
 				ISC_INT64 n = *(ISC_INT64 *) data, f = scales[-scale];
 
 				if (n >= 0) {
-					l = sprintf(string_data, "%" LL_MASK "d.%0*" LL_MASK "d", n / f, -scale, n % f);
+					l = slprintf(string_data, sizeof(string_data), "%" LL_MASK "d.%0*" LL_MASK "d", n / f, -scale, n % f);
 				} else if (n <= -f) {
-					l = sprintf(string_data, "%" LL_MASK "d.%0*" LL_MASK "d", n / f, -scale, -n % f);				
+					l = slprintf(string_data, sizeof(string_data), "%" LL_MASK "d.%0*" LL_MASK "d", n / f, -scale, -n % f);				
 				 } else {
-					l = sprintf(string_data, "-0.%0*" LL_MASK "d", -scale, -n % f);
+					l = slprintf(string_data, sizeof(string_data), "-0.%0*" LL_MASK "d", -scale, -n % f);
 				}
 				ZVAL_STRINGL(val,string_data,l,1);
 			}
@@ -1342,11 +1366,11 @@ static int _php_ibase_var_zval(zval *val, void *data, int type, int len, /* {{{
 				long f = (long) scales[-scale];
 
 				if (n >= 0) {
-					l = sprintf(string_data, "%ld.%0*ld", n / f, -scale,  n % f);
+					l = slprintf(string_data, sizeof(string_data), "%ld.%0*ld", n / f, -scale,  n % f);
 				} else if (n <= -f) {
-					l = sprintf(string_data, "%ld.%0*ld", n / f, -scale,  -n % f);
+					l = slprintf(string_data, sizeof(string_data), "%ld.%0*ld", n / f, -scale,  -n % f);
 				} else {
-					l = sprintf(string_data, "-0.%0*ld", -scale, -n % f);
+					l = slprintf(string_data, sizeof(string_data), "-0.%0*ld", -scale, -n % f);
 				}
 				ZVAL_STRINGL(val,string_data,l,1);
 			}
@@ -1386,14 +1410,14 @@ format_date_time:
 #else
 				switch (type & ~1) {
 					default:
-						l = sprintf(string_data, "%02d/%02d/%4d %02d:%02d:%02d", t.tm_mon+1, t.tm_mday, 
+						l = slprintf(string_data, sizeof(string_data), "%02d/%02d/%4d %02d:%02d:%02d", t.tm_mon+1, t.tm_mday, 
 							t.tm_year + 1900, t.tm_hour, t.tm_min, t.tm_sec);
 						break;
 					case SQL_TYPE_DATE:
-						l = sprintf(string_data, "%02d/%02d/%4d", t.tm_mon + 1, t.tm_mday, t.tm_year+1900);
+						l = slprintf(string_data, sizeof(string_data), "%02d/%02d/%4d", t.tm_mon + 1, t.tm_mday, t.tm_year+1900);
 						break;
 					case SQL_TYPE_TIME:
-						l = sprintf(string_data, "%02d:%02d:%02d", t.tm_hour, t.tm_min, t.tm_sec);
+						l = slprintf(string_data, sizeof(string_data), "%02d:%02d:%02d", t.tm_hour, t.tm_min, t.tm_sec);
 						break;
 				}
 #endif
@@ -1456,40 +1480,24 @@ static int _php_ibase_arr_zval(zval *ar_zval, char *data, unsigned long data_siz
 
 static void _php_ibase_fetch_hash(INTERNAL_FUNCTION_PARAMETERS, int fetch_type) /* {{{ */
 {
-	zval **result_arg, **flag_arg;
+	zval *result_arg;
 	long i, array_cnt = 0, flag = 0;
 	ibase_result *ib_result;
 
 	RESET_ERRMSG;
 
-	switch (ZEND_NUM_ARGS()) {
-		case 1:
-			if (FAILURE == zend_get_parameters_ex(1, &result_arg)) {
-				RETURN_FALSE;
-			}
-			break;
-		case 2:
-			if (FAILURE == zend_get_parameters_ex(2, &result_arg, &flag_arg)) {
-				RETURN_FALSE;
-			}
-			convert_to_long_ex(flag_arg);
-			flag = Z_LVAL_PP(flag_arg);
-			break;
-		default:
-			WRONG_PARAM_COUNT;
-			break;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|l", &result_arg, &flag)) {
+		return;
 	}
 
-	ZEND_FETCH_RESOURCE(ib_result, ibase_result *, result_arg, -1, LE_RESULT, le_result);
+	ZEND_FETCH_RESOURCE(ib_result, ibase_result *, &result_arg, -1, LE_RESULT, le_result);
 
 	if (ib_result->out_sqlda == NULL || !ib_result->has_more_rows) {
 		RETURN_FALSE;
 	}
 
 	if (ib_result->statement_type != isc_info_sql_stmt_exec_procedure) {
-
 		if (isc_dsql_fetch(IB_STATUS, &ib_result->stmt, 1, ib_result->out_sqlda)) {
-
 			ib_result->has_more_rows = 0;
 			if (IB_STATUS[0] && IB_STATUS[1]) { /* error in fetch */
 				_php_ibase_error(TSRMLS_C);
@@ -1525,7 +1533,7 @@ static void _php_ibase_fetch_hash(INTERNAL_FUNCTION_PARAMETERS, int fetch_type)
 							Z_ARRVAL_P(return_value),alias,strlen(alias)+1,&p)) {
 				
 				case '\0':
-						sprintf(alias = buf, "%s_%02d", base, i++);
+						snprintf(alias = buf, sizeof(buf), "%s_%02d", base, i++);
 					}
 			}
 		}
@@ -1684,19 +1692,20 @@ PHP_FUNCTION(ibase_fetch_object)
    Assign a name to a result for use with ... WHERE CURRENT OF <name> statements */
 PHP_FUNCTION(ibase_name_result)
 {
-	zval **result_arg, **name_arg;
+	zval *result_arg;
+	char *name_arg;
+	int name_arg_len;
 	ibase_result *ib_result;
 
 	RESET_ERRMSG;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &result_arg, &name_arg) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs", &result_arg, &name_arg, &name_arg_len) == FAILURE) {
+		return;
 	}
 
-	ZEND_FETCH_RESOURCE(ib_result, ibase_result *, result_arg, -1, LE_RESULT, le_result);
-	convert_to_string_ex(name_arg);
+	ZEND_FETCH_RESOURCE(ib_result, ibase_result *, &result_arg, -1, LE_RESULT, le_result);
 
-	if (isc_dsql_set_cursor_name(IB_STATUS, &ib_result->stmt, Z_STRVAL_PP(name_arg), 0)) {
+	if (isc_dsql_set_cursor_name(IB_STATUS, &ib_result->stmt, name_arg, 0)) {
 		_php_ibase_error(TSRMLS_C);
 		RETURN_FALSE;
 	}
@@ -1709,81 +1718,64 @@ PHP_FUNCTION(ibase_name_result)
    Free the memory used by a result */
 PHP_FUNCTION(ibase_free_result)
 {
-	zval **result_arg;
+	zval *result_arg;
 	ibase_result *ib_result;
 
 	RESET_ERRMSG;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &result_arg) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &result_arg) == FAILURE) {
+		return;
 	}
 
-	ZEND_FETCH_RESOURCE(ib_result, ibase_result *, result_arg, -1, LE_RESULT, le_result);
-	zend_list_delete(Z_LVAL_PP(result_arg));
+	ZEND_FETCH_RESOURCE(ib_result, ibase_result *, &result_arg, -1, LE_RESULT, le_result);
+	zend_list_delete(Z_RESVAL_P(result_arg));
 	RETURN_TRUE;
 }
 /* }}} */
 
-/* {{{ proto resource ibase_prepare([resource link_identifier, ] string query)
+/* {{{ proto resource ibase_prepare(resource link_identifier[, string query [, resource trans_identifier ]])
    Prepare a query for later execution */
 PHP_FUNCTION(ibase_prepare)
 {
-	zval **link_arg, **trans_arg, **query_arg;
+	zval *link_arg, *trans_arg;
 	ibase_db_link *ib_link;
 	ibase_trans *trans = NULL;
-	int trans_res_id = 0;
+	int query_len, trans_res_id = 0;
 	ibase_query *ib_query;
 	char *query;
 
 	RESET_ERRMSG;
+	
+	if (ZEND_NUM_ARGS() == 1) {
+		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &query, &query_len) == FAILURE) {
+			return;
+		}
+		ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, NULL, IBG(default_link), LE_LINK, le_link, le_plink);
+	} else if (ZEND_NUM_ARGS() == 2) {
+		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs", &link_arg, &query, &query_len) == FAILURE) {
+			return;
+		}
+		_php_ibase_get_link_trans(INTERNAL_FUNCTION_PARAM_PASSTHRU, &link_arg, &ib_link, &trans);
 
-	switch (ZEND_NUM_ARGS()) {
-		case 1:
-			if (zend_get_parameters_ex(1, &query_arg) == FAILURE) {
-				RETURN_FALSE;
-			}
-			ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, NULL, IBG(default_link), LE_LINK,
-				le_link, le_plink);
-			break;
-		case 2:
-			if (zend_get_parameters_ex(2, &link_arg, &query_arg) == FAILURE) {
-				RETURN_FALSE;
-			}
-			_php_ibase_get_link_trans(INTERNAL_FUNCTION_PARAM_PASSTHRU, link_arg, &ib_link, &trans);
-
-			if (trans != NULL) {
-				convert_to_long_ex(link_arg);
-				trans_res_id = Z_LVAL_PP(link_arg);
-			}
-
-			break;
-		case 3:
-			if (zend_get_parameters_ex(3, &link_arg, &trans_arg, &query_arg) == FAILURE) {
-				RETURN_FALSE;
-			}
-			ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link*, link_arg, -1, LE_LINK, le_link, le_plink);
-			ZEND_FETCH_RESOURCE(trans, ibase_trans*, trans_arg, -1, LE_TRANS, le_trans);
-
-			convert_to_long_ex(trans_arg);
-			trans_res_id = Z_LVAL_PP(trans_arg);
-
-			break;			
-		default:
-			WRONG_PARAM_COUNT;
-			break;
+		if (trans != NULL) {
+			trans_res_id = Z_RESVAL_P(link_arg);
+		}
+	} else {
+		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rrs", &link_arg, &trans_arg, &query, &query_len) == FAILURE) {
+			return;
+		}
+		ZEND_FETCH_RESOURCE2(ib_link, ibase_db_link *, &link_arg, -1, LE_LINK, le_link, le_plink);
+		ZEND_FETCH_RESOURCE(trans, ibase_trans *, &trans_arg, -1, LE_TRANS, le_trans);
+		trans_res_id = Z_RESVAL_P(trans_arg);
 	}
 
-	convert_to_string_ex(query_arg);
-	query = Z_STRVAL_PP(query_arg);
-
 	if (FAILURE == _php_ibase_def_trans(ib_link, &trans TSRMLS_CC)) {
 		RETURN_FALSE;
 	}
 
 	ib_query = (ibase_query *) emalloc(sizeof(ibase_query));
 
-	if (FAILURE == _php_ibase_alloc_query(ib_query, ib_link, trans, query, ib_link->dialect,
-			trans_res_id TSRMLS_CC)) {
+	if (FAILURE == _php_ibase_alloc_query(ib_query, ib_link, trans, query, ib_link->dialect, trans_res_id TSRMLS_CC)) {
 		efree(ib_query);
 		RETURN_FALSE;
 	}
@@ -1798,6 +1790,7 @@ PHP_FUNCTION(ibase_execute)
 	zval *query, ***args = NULL;
 	ibase_query *ib_query;
 	ibase_result *result = NULL;
+	ALLOCA_FLAG(use_heap)
 
 	RESET_ERRMSG;
 	
@@ -1816,16 +1809,17 @@ PHP_FUNCTION(ibase_execute)
 		if (bind_n != expected_n) {
 			php_error_docref(NULL TSRMLS_CC, (bind_n < expected_n) ? E_WARNING : E_NOTICE,
 				"Statement expects %d arguments, %d given", expected_n, bind_n);
+
 			if (bind_n < expected_n) {
 				break;
 			}
+		}
 
-		} else if (bind_n > 0) { /* have variables to bind */
-			args = (zval ***) do_alloca(ZEND_NUM_ARGS() * sizeof(zval **));
+		/* have variables to bind */
+		args = (zval ***) do_alloca((expected_n + 1) * sizeof(zval **), use_heap);
 	
-			if (FAILURE == zend_get_parameters_array_ex(ZEND_NUM_ARGS(), args)) {
-				break;
-			}
+		if (FAILURE == zend_get_parameters_array_ex((expected_n + 1), args)) {
+			break;
 		}
 
 		/* Have we used this cursor before and it's still open (exec proc has no cursor) ? */
@@ -1862,7 +1856,7 @@ PHP_FUNCTION(ibase_execute)
 	} while (0);
 
 	if (args) {
-		free_alloca(args);
+		free_alloca(args, use_heap);
 	}
 }
 /* }}} */
@@ -1871,17 +1865,17 @@ PHP_FUNCTION(ibase_execute)
    Free memory used by a query */
 PHP_FUNCTION(ibase_free_query)
 {
-	zval **query_arg;
+	zval *query_arg;
 	ibase_query *ib_query;
 
 	RESET_ERRMSG;
 
-	if (ZEND_NUM_ARGS()!=1 || zend_get_parameters_ex(1, &query_arg) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &query_arg) == FAILURE) {
+		return;
 	}
 
-	ZEND_FETCH_RESOURCE(ib_query, ibase_query *, query_arg, -1, LE_QUERY, le_query);
-	zend_list_delete(Z_LVAL_PP(query_arg));
+	ZEND_FETCH_RESOURCE(ib_query, ibase_query *, &query_arg, -1, LE_QUERY, le_query);
+	zend_list_delete(Z_RESVAL_P(query_arg));
 	RETURN_TRUE;
 }
 /* }}} */
@@ -1890,29 +1884,29 @@ PHP_FUNCTION(ibase_free_query)
    Get the number of fields in result */
 PHP_FUNCTION(ibase_num_fields)
 {
-	zval **result;
+	zval *result;
 	int type;
 	XSQLDA *sqlda;
 
 	RESET_ERRMSG;
 
-	if (ZEND_NUM_ARGS()!=1 || zend_get_parameters_ex(1, &result)==FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &result) == FAILURE) {
+		return;
 	}
 
-	zend_list_find(Z_LVAL_PP(result), &type);
+	zend_list_find(Z_RESVAL_P(result), &type);
 
 	if (type == le_query) {
 		ibase_query *ib_query;
 
-		ZEND_FETCH_RESOURCE(ib_query, ibase_query *, result, -1, LE_QUERY, le_query);
+		ZEND_FETCH_RESOURCE(ib_query, ibase_query *, &result, -1, LE_QUERY, le_query);
 		sqlda = ib_query->out_sqlda;
 	} else {
 		ibase_result *ib_result;
 
-		ZEND_FETCH_RESOURCE(ib_result, ibase_result *, result, -1, LE_RESULT, le_result);
+		ZEND_FETCH_RESOURCE(ib_result, ibase_result *, &result, -1, LE_RESULT, le_result);
 		sqlda = ib_result->out_sqlda;
-	}					
+	}
 
 	if (sqlda == NULL) {
 		RETURN_LONG(0);
@@ -1938,7 +1932,7 @@ static void _php_ibase_field_info(zval *return_value, XSQLVAR *var) /* {{{ */
 	add_index_stringl(return_value, 2, var->relname, var->relname_length, 1);
 	add_assoc_stringl(return_value, "relation", var->relname, var->relname_length, 1);
 
-	len = snprintf(buf, 16, "%d", var->sqllen);
+	len = slprintf(buf, 16, "%d", var->sqllen);
 	add_index_stringl(return_value, 3, buf, len, 1);
 	add_assoc_stringl(return_value, "length", buf, len, 1);
 
@@ -1957,7 +1951,7 @@ static void _php_ibase_field_info(zval *return_value, XSQLVAR *var) /* {{{ */
 				precision = 18;
 				break;
 		}
-		len = snprintf(buf, 16, "NUMERIC(%d,%d)", precision, -var->sqlscale);
+		len = slprintf(buf, 16, "NUMERIC(%d,%d)", precision, -var->sqlscale);
 		add_index_stringl(return_value, 4, s, len, 1);
 		add_assoc_stringl(return_value, "type", s, len, 1);
 	} else {
@@ -2013,27 +2007,28 @@ static void _php_ibase_field_info(zval *return_value, XSQLVAR *var) /* {{{ */
    Get information about a field */
 PHP_FUNCTION(ibase_field_info)
 {
-	zval **result_arg, **field_arg;
+	zval *result_arg;
+	long field_arg;
 	int type;
 	XSQLDA *sqlda;
 
 	RESET_ERRMSG;
 
-	if (ZEND_NUM_ARGS()!=2 || zend_get_parameters_ex(2, &result_arg, &field_arg)==FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &result_arg, &field_arg) == FAILURE) {
+		return;
 	}
 
-	zend_list_find(Z_LVAL_PP(result_arg), &type);
+	zend_list_find(Z_RESVAL_P(result_arg), &type);
 
 	if (type == le_query) {
 		ibase_query *ib_query;
 
-		ZEND_FETCH_RESOURCE(ib_query, ibase_query *, result_arg, -1, LE_QUERY, le_query);
+		ZEND_FETCH_RESOURCE(ib_query, ibase_query *, &result_arg, -1, LE_QUERY, le_query);
 		sqlda = ib_query->out_sqlda;
 	} else {
 		ibase_result *ib_result;
 
-		ZEND_FETCH_RESOURCE(ib_result, ibase_result *, result_arg, -1, LE_RESULT, le_result);
+		ZEND_FETCH_RESOURCE(ib_result, ibase_result *, &result_arg, -1, LE_RESULT, le_result);
 		sqlda = ib_result->out_sqlda;
 	}					
 
@@ -2042,12 +2037,10 @@ PHP_FUNCTION(ibase_field_info)
 		RETURN_FALSE;
 	}
 
-	convert_to_long_ex(field_arg);
-
-	if (Z_LVAL_PP(field_arg) < 0 || Z_LVAL_PP(field_arg) >= sqlda->sqld) {
+	if (field_arg < 0 || field_arg >= sqlda->sqld) {
 		RETURN_FALSE;
 	}
-	_php_ibase_field_info(return_value,sqlda->sqlvar + Z_LVAL_PP(field_arg));
+	_php_ibase_field_info(return_value, sqlda->sqlvar + field_arg);
 }
 /* }}} */
 
@@ -2055,16 +2048,16 @@ PHP_FUNCTION(ibase_field_info)
    Get the number of params in a prepared query */
 PHP_FUNCTION(ibase_num_params)
 {
-	zval **result;
+	zval *result;
 	ibase_query *ib_query;
 
 	RESET_ERRMSG;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &result) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &result) == FAILURE) {
+		return;
 	}
 
-	ZEND_FETCH_RESOURCE(ib_query, ibase_query *, result, -1, LE_QUERY, le_query);
+	ZEND_FETCH_RESOURCE(ib_query, ibase_query *, &result, -1, LE_QUERY, le_query);
 
 	if (ib_query->in_sqlda == NULL) {
 		RETURN_LONG(0);
@@ -2078,28 +2071,27 @@ PHP_FUNCTION(ibase_num_params)
    Get information about a parameter */
 PHP_FUNCTION(ibase_param_info)
 {
-	zval **result_arg, **field_arg;
+	zval *result_arg;
+	long field_arg;
 	ibase_query *ib_query;
 
 	RESET_ERRMSG;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &result_arg, &field_arg) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &result_arg, &field_arg) == FAILURE) {
+		return;
 	}
 
-	ZEND_FETCH_RESOURCE(ib_query, ibase_query *, result_arg, -1, LE_QUERY, le_query);
+	ZEND_FETCH_RESOURCE(ib_query, ibase_query *, &result_arg, -1, LE_QUERY, le_query);
 
 	if (ib_query->in_sqlda == NULL) {
 		RETURN_FALSE;
 	}
 
-	convert_to_long_ex(field_arg);
-
-	if (Z_LVAL_PP(field_arg) < 0 || Z_LVAL_PP(field_arg) >= ib_query->in_sqlda->sqld) {
+	if (field_arg < 0 || field_arg >= ib_query->in_sqlda->sqld) {
 		RETURN_FALSE;
 	}
 
-	_php_ibase_field_info(return_value,ib_query->in_sqlda->sqlvar + Z_LVAL_PP(field_arg));
+	_php_ibase_field_info(return_value,ib_query->in_sqlda->sqlvar + field_arg);
 }
 /* }}} */
 
