@@ -15,7 +15,7 @@
    | Author: Pierre A. Joye <pierre@php.net>                              |
    +----------------------------------------------------------------------+
  */
-/* $Id: idn.c 292566 2009-12-23 21:41:05Z stas $ */
+/* $Id: idn.c 314218 2011-08-04 00:59:43Z felipe $ */
 
 /* {{{ includes */
 #ifdef HAVE_CONFIG_H
@@ -85,7 +85,9 @@ static void php_intl_idn_to(INTERNAL_FUNCTION_PARAMETERS, int mode)
 
 		/* Set error messages. */
 		intl_error_set_custom_msg( NULL, "Error converting input string to UTF-16", 0 TSRMLS_CC );
-		efree(ustring);
+		if (ustring) {
+			efree(ustring);
+		}
 		RETURN_FALSE;
 	} else {
 		UParseError parse_error;
