@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: php_mbregex.c 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id: php_mbregex.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -1245,14 +1245,19 @@ PHP_FUNCTION(mb_ereg_search_init)
 {
 	size_t argc = ZEND_NUM_ARGS();
 	zval *arg_str;
-	char *arg_pattern, *arg_options;
-	int arg_pattern_len, arg_options_len;
+	char *arg_pattern = NULL, *arg_options = NULL;
+	int arg_pattern_len = 0, arg_options_len = 0;
 	OnigSyntaxType *syntax = NULL;
 	OnigOptionType option;
 
 	if (zend_parse_parameters(argc TSRMLS_CC, "z|ss", &arg_str, &arg_pattern, &arg_pattern_len, &arg_options, &arg_options_len) == FAILURE) {
 		return;
 	}
+	
+	if (argc > 1 && arg_pattern_len == 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Empty pattern");
+		RETURN_FALSE;
+	}
 
 	option = MBREX(regex_default_options);
 	syntax = MBREX(regex_default_syntax);
