@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
  */
 
 
-/* $Id: php_mssql.h 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id: php_mssql.h 321634 2012-01-01 13:15:04Z felipe $ */
 
 #ifndef PHP_MSSQL_H
 #define PHP_MSSQL_H
@@ -26,6 +26,11 @@
 #if HAVE_MSSQL
 #define MSDBLIB
 
+/* FreeTDS checks for PHP_MSSQL_API for avoid type redefinition */
+#ifdef HAVE_FREETDS
+#define PHP_MSSQL_API
+#endif
+
 #include <sqlfront.h>
 #include <sqldb.h>
 
