@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006-2010 The PHP Group                                |
+  | Copyright (c) 2006-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -18,7 +18,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: php_mysqlnd.c 300352 2010-06-10 12:24:03Z andrey $ */
+/* $Id: php_mysqlnd.c 321634 2012-01-01 13:15:04Z felipe $ */
 #include "php.h"
 #include "php_ini.h"
 #include "mysqlnd.h"
@@ -31,13 +31,13 @@
  * Every user visible function must have an entry in mysqlnd_functions[].
  */
 static zend_function_entry mysqlnd_functions[] = {
-	{NULL, NULL, NULL}	/* Must be the last line in mysqlnd_functions[] */
+	PHP_FE_END
 };
 /* }}} */
 
 
 /* {{{ mysqlnd_minfo_print_hash */
-#if PHP_MAJOR_VERSION >= 6
+#if MYSQLND_UNICODE
 PHPAPI void mysqlnd_minfo_print_hash(zval *values)
 {
 	zval **values_entry;
@@ -122,6 +122,8 @@ PHP_MINFO_FUNCTION(mysqlnd)
 	php_info_print_table_row(2, "Read timeout", buf);
 	php_info_print_table_row(2, "Collecting statistics", MYSQLND_G(collect_statistics)? "Yes":"No");
 	php_info_print_table_row(2, "Collecting memory statistics", MYSQLND_G(collect_memory_statistics)? "Yes":"No");
+
+	php_info_print_table_row(2, "Tracing", MYSQLND_G(debug)? MYSQLND_G(debug):"n/a");
 	php_info_print_table_end();
 
 	/* Print client stats */
@@ -136,7 +138,7 @@ PHP_MINFO_FUNCTION(mysqlnd)
 /* }}} */
 
 
-PHPAPI ZEND_DECLARE_MODULE_GLOBALS(mysqlnd);
+PHPAPI ZEND_DECLARE_MODULE_GLOBALS(mysqlnd)
 
 
 /* {{{ PHP_GINIT_FUNCTION
@@ -264,7 +266,7 @@ static PHP_RSHUTDOWN_FUNCTION(mysqlnd)
 
 static const zend_module_dep mysqlnd_deps[] = {
 	ZEND_MOD_REQUIRED("standard")
-	{NULL, NULL, NULL}
+	ZEND_MOD_END
 };
 
 /* {{{ mysqlnd_module_entry
@@ -297,6 +299,11 @@ zend_module_entry mysqlnd_module_entry = {
 };
 /* }}} */
 
+/* {{{ COMPILE_DL_MYSQLND */
+#ifdef COMPILE_DL_MYSQLND
+ZEND_GET_MODULE(mysqlnd)
+#endif
+/* }}} */
 
 /*
  * Local variables:
