@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2006 The PHP Group                                |
+  | Copyright (c) 1997-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: php_pdo_firebird_int.h,v 1.10.2.1 2006/01/01 12:50:11 sniper Exp $ */
+/* $Id: php_pdo_firebird_int.h 321634 2012-01-01 13:15:04Z felipe $ */
 
 #ifndef PHP_PDO_FIREBIRD_INT_H
 #define PHP_PDO_FIREBIRD_INT_H
@@ -31,6 +31,10 @@
 
 #define PDO_FB_DIALECT 3
 
+#define PDO_FB_DEF_DATE_FMT "%Y-%m-%d"
+#define PDO_FB_DEF_TIME_FMT "%H:%M:%S"
+#define PDO_FB_DEF_TIMESTAMP_FMT PDO_FB_DEF_DATE_FMT " " PDO_FB_DEF_TIME_FMT
+
 #define SHORT_MAX (1 << (8*sizeof(short)-1))
 
 #if SIZEOF_LONG == 8
@@ -73,6 +77,16 @@ typedef struct {
 	/* the last error that didn't come from the API */
 	char const *last_app_error;
 	
+	/* date and time format strings, can be set by the set_attribute method */
+	char *date_format;
+	char *time_format;
+	char *timestamp_format;
+	
+	/* prepend table names on column names in fetch */
+	unsigned fetch_table_names:1;
+	
+	unsigned _reserved:31;
+	
 } pdo_firebird_db_handle;
 
 
@@ -93,7 +107,10 @@ typedef struct {
 	/* whether EOF was reached for this statement */
 	unsigned exhausted:1;
 
-	unsigned _reserved:23;
+	/* successful isc_dsql_execute opens a cursor */
+	unsigned cursor_open:1;
+
+	unsigned _reserved:22;
 
 	/* the named params that were converted to ?'s by the driver */
 	HashTable *named_params;
@@ -115,6 +132,12 @@ extern struct pdo_stmt_methods firebird_stmt_methods;
 
 void _firebird_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, char const *file, long line TSRMLS_DC);
 
+enum {
+	PDO_FB_ATTR_DATE_FORMAT = PDO_ATTR_DRIVER_SPECIFIC,
+	PDO_FB_ATTR_TIME_FORMAT,
+	PDO_FB_ATTR_TIMESTAMP_FORMAT,
+};
+
 #endif	/* PHP_PDO_FIREBIRD_INT_H */
 
 /*
