@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2010 The PHP Group                                |
+  | Copyright (c) 1997-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: sqlite_driver.c 300612 2010-06-20 14:12:06Z felipe $ */
+/* $Id: sqlite_driver.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -47,33 +47,33 @@ int _pdo_sqlite_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, const char *file, int li
 		}
 		einfo->errmsg = pestrdup((char*)sqlite3_errmsg(H->db), dbh->is_persistent);
 	} else { /* no error */
-		strcpy(*pdo_err, PDO_ERR_NONE);
+		strncpy(*pdo_err, PDO_ERR_NONE, sizeof(PDO_ERR_NONE));
 		return 0;
 	}
 	switch (einfo->errcode) {
 		case SQLITE_NOTFOUND:
-			strcpy(*pdo_err, "42S02");
+			strncpy(*pdo_err, "42S02", sizeof("42S02"));
 			break;	
 
 		case SQLITE_INTERRUPT:
-			strcpy(*pdo_err, "01002");
+			strncpy(*pdo_err, "01002", sizeof("01002"));
 			break;
 
 		case SQLITE_NOLFS:
-			strcpy(*pdo_err, "HYC00");
+			strncpy(*pdo_err, "HYC00", sizeof("HYC00"));
 			break;
 
 		case SQLITE_TOOBIG:
-			strcpy(*pdo_err, "22001");
+			strncpy(*pdo_err, "22001", sizeof("22001"));
 			break;
 	
 		case SQLITE_CONSTRAINT:
-			strcpy(*pdo_err, "23000");
+			strncpy(*pdo_err, "23000", sizeof("23000"));
 			break;
 
 		case SQLITE_ERROR:
 		default:
-			strcpy(*pdo_err, "HY000");
+			strncpy(*pdo_err, "HY000", sizeof("HY000"));
 			break;
 	}
 
@@ -593,7 +593,7 @@ static PHP_METHOD(SQLite, sqliteCreateAggregate)
 static const zend_function_entry dbh_methods[] = {
 	PHP_ME(SQLite, sqliteCreateFunction, NULL, ZEND_ACC_PUBLIC)
 	PHP_ME(SQLite, sqliteCreateAggregate, NULL, ZEND_ACC_PUBLIC)
-	{NULL, NULL, NULL}
+	PHP_FE_END
 };
 
 static const zend_function_entry *get_driver_methods(pdo_dbh_t *dbh, int kind TSRMLS_DC)
