@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
  
-/* $Id: recode.c 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id: recode.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 /* {{{ includes & prototypes */
 
@@ -84,7 +84,7 @@ static const zend_function_entry php_recode_functions[] = {
 	PHP_FE(recode_string, 	arginfo_recode_string)
 	PHP_FE(recode_file, 	arginfo_recode_file)
 	PHP_FALIAS(recode, recode_string, arginfo_recode_string)
-	{NULL, NULL, NULL}
+	PHP_FE_END
 };
 
 zend_module_entry recode_module_entry = {
@@ -135,7 +135,7 @@ PHP_MINFO_FUNCTION(recode)
 {
 	php_info_print_table_start();
 	php_info_print_table_row(2, "Recode Support", "enabled");
-	php_info_print_table_row(2, "Revision", "$Revision: 293036 $");
+	php_info_print_table_row(2, "Revision", "$Revision: 321634 $");
 	php_info_print_table_end();
 }
 
