@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: session.c 305034 2010-11-02 18:34:56Z felipe $ */
+/* $Id: session.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -1742,7 +1742,7 @@ static PHP_FUNCTION(session_regenerate_id)
 		return;
 	}
 
-	if (SG(headers_sent)) {
+	if (SG(headers_sent) && PS(use_cookies)) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot regenerate session id - headers already sent");
 		RETURN_FALSE;
 	}
@@ -1914,10 +1914,10 @@ static PHP_FUNCTION(session_unset)
 	}
 
 	IF_SESSION_VARS() {
-		HashTable *ht;
+		HashTable *ht_sess_var;
 
 		SEPARATE_ZVAL_IF_NOT_REF(&PS(http_session_vars));
-		ht = Z_ARRVAL_P(PS(http_session_vars));
+		ht_sess_var = Z_ARRVAL_P(PS(http_session_vars));
 
 		if (PG(register_globals)) {
 			uint str_len;
@@ -1925,16 +1925,16 @@ static PHP_FUNCTION(session_unset)
 			ulong num_key;
 			HashPosition pos;
 
-			zend_hash_internal_pointer_reset_ex(ht, &pos);
+			zend_hash_internal_pointer_reset_ex(ht_sess_var, &pos);
 
-			while (zend_hash_get_current_key_ex(ht, &str, &str_len, &num_key, 0, &pos) == HASH_KEY_IS_STRING) {
+			while (zend_hash_get_current_key_ex(ht_sess_var, &str, &str_len, &num_key, 0, &pos) == HASH_KEY_IS_STRING) {
 				zend_delete_global_variable(str, str_len - 1 TSRMLS_CC);
-				zend_hash_move_forward_ex(ht, &pos);
+				zend_hash_move_forward_ex(ht_sess_var, &pos);
 			}
 		}
 
 		/* Clean $_SESSION. */
-		zend_hash_clean(ht);
+		zend_hash_clean(ht_sess_var);
 	}
 }
 /* }}} */
@@ -2119,7 +2119,7 @@ static const zend_function_entry session_functions[] = {
 	PHP_FE(session_get_cookie_params, arginfo_session_void)
 	PHP_FE(session_write_close,       arginfo_session_void)
 	PHP_FALIAS(session_commit, session_write_close, arginfo_session_void)
-	{NULL, NULL, NULL}
+	PHP_FE_END
 };
 /* }}} */
 
@@ -2283,7 +2283,7 @@ static PHP_MINFO_FUNCTION(session) /* {{{ */
 static const zend_module_dep session_deps[] = { /* {{{ */
 	ZEND_MOD_OPTIONAL("hash")
 	ZEND_MOD_REQUIRED("spl")
-	{NULL, NULL, NULL}
+	ZEND_MOD_END
 };
 /* }}} */
 
