@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: sxe.c 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id: sxe.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -41,6 +41,10 @@ zend_class_entry *ce_SimpleXMLElement;
 PHP_METHOD(ce_SimpleXMLIterator, rewind)
 {
 	php_sxe_iterator iter;
+	
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
 
 	iter.sxe = php_sxe_fetch_object(getThis() TSRMLS_CC);
 	ce_SimpleXMLElement->iterator_funcs.funcs->rewind((zend_object_iterator*)&iter TSRMLS_CC);
@@ -52,6 +56,10 @@ PHP_METHOD(ce_SimpleXMLIterator, rewind)
 PHP_METHOD(ce_SimpleXMLIterator, valid)
 {
 	php_sxe_object *sxe = php_sxe_fetch_object(getThis() TSRMLS_CC);
+	
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
 
 	RETURN_BOOL(sxe->iter.data);
 }
@@ -62,6 +70,10 @@ PHP_METHOD(ce_SimpleXMLIterator, valid)
 PHP_METHOD(ce_SimpleXMLIterator, current)
 {
 	php_sxe_object *sxe = php_sxe_fetch_object(getThis() TSRMLS_CC);
+	
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
 
 	if (!sxe->iter.data) {
 		return; /* return NULL */
@@ -78,6 +90,10 @@ PHP_METHOD(ce_SimpleXMLIterator, key)
 	xmlNodePtr curnode;
 	php_sxe_object *intern;
 	php_sxe_object *sxe = php_sxe_fetch_object(getThis() TSRMLS_CC);
+	
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
 
 	if (!sxe->iter.data) {
 		RETURN_FALSE;
@@ -98,6 +114,10 @@ PHP_METHOD(ce_SimpleXMLIterator, key)
 PHP_METHOD(ce_SimpleXMLIterator, next)
 {
 	php_sxe_iterator iter;
+	
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
 
 	iter.sxe = php_sxe_fetch_object(getThis() TSRMLS_CC);
 	ce_SimpleXMLElement->iterator_funcs.funcs->move_forward((zend_object_iterator*)&iter TSRMLS_CC);
@@ -111,6 +131,10 @@ PHP_METHOD(ce_SimpleXMLIterator, hasChildren)
 	php_sxe_object *sxe = php_sxe_fetch_object(getThis() TSRMLS_CC);
 	php_sxe_object *child;
 	xmlNodePtr      node;
+	
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
 
 	if (!sxe->iter.data || sxe->iter.type == SXE_ITER_ATTRLIST) {
 		RETURN_FALSE;
@@ -133,6 +157,10 @@ PHP_METHOD(ce_SimpleXMLIterator, hasChildren)
 PHP_METHOD(ce_SimpleXMLIterator, getChildren)
 {
 	php_sxe_object *sxe = php_sxe_fetch_object(getThis() TSRMLS_CC);
+	
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
 
 	if (!sxe->iter.data || sxe->iter.type == SXE_ITER_ATTRLIST) {
 		return; /* return NULL */
@@ -140,14 +168,19 @@ PHP_METHOD(ce_SimpleXMLIterator, getChildren)
 	RETURN_ZVAL(sxe->iter.data, 1, 0);
 }
 
+/* {{{ arginfo */
+ZEND_BEGIN_ARG_INFO(arginfo_simplexmliterator__void, 0)
+ZEND_END_ARG_INFO()
+/* }}} */
+
 static const zend_function_entry funcs_SimpleXMLIterator[] = {
-	PHP_ME(ce_SimpleXMLIterator, rewind,                 NULL, ZEND_ACC_PUBLIC)
-	PHP_ME(ce_SimpleXMLIterator, valid,                  NULL, ZEND_ACC_PUBLIC)
-	PHP_ME(ce_SimpleXMLIterator, current,                NULL, ZEND_ACC_PUBLIC)
-	PHP_ME(ce_SimpleXMLIterator, key,                    NULL, ZEND_ACC_PUBLIC)
-	PHP_ME(ce_SimpleXMLIterator, next,                   NULL, ZEND_ACC_PUBLIC)
-	PHP_ME(ce_SimpleXMLIterator, hasChildren,            NULL, ZEND_ACC_PUBLIC)
-	PHP_ME(ce_SimpleXMLIterator, getChildren,            NULL, ZEND_ACC_PUBLIC)
+	PHP_ME(ce_SimpleXMLIterator, rewind,                 arginfo_simplexmliterator__void, ZEND_ACC_PUBLIC)
+	PHP_ME(ce_SimpleXMLIterator, valid,                  arginfo_simplexmliterator__void, ZEND_ACC_PUBLIC)
+	PHP_ME(ce_SimpleXMLIterator, current,                arginfo_simplexmliterator__void, ZEND_ACC_PUBLIC)
+	PHP_ME(ce_SimpleXMLIterator, key,                    arginfo_simplexmliterator__void, ZEND_ACC_PUBLIC)
+	PHP_ME(ce_SimpleXMLIterator, next,                   arginfo_simplexmliterator__void, ZEND_ACC_PUBLIC)
+	PHP_ME(ce_SimpleXMLIterator, hasChildren,            arginfo_simplexmliterator__void, ZEND_ACC_PUBLIC)
+	PHP_ME(ce_SimpleXMLIterator, getChildren,            arginfo_simplexmliterator__void, ZEND_ACC_PUBLIC)
 	{NULL, NULL, NULL}
 };
 /* }}} */
