@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2007 The PHP Group                                |
+  | Copyright (c) 1997-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
   |          Dmitry Stogov <dmitry@zend.com>                             |
   +----------------------------------------------------------------------+
 */
-/* $Id: php_schema.c,v 1.58.2.6.2.5 2007/02/15 17:01:29 dmitry Exp $ */
+/* $Id: php_schema.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #include "php_soap.h"
 #include "libxml/uri.h"
@@ -102,7 +102,10 @@ static void schema_load_file(sdlCtx *ctx, xmlAttrPtr ns, xmlChar *location, xmlA
 		xmlNodePtr schema;
 		xmlAttrPtr new_tns;
 
+		sdl_set_uri_credentials(ctx, (char*)location TSRMLS_CC);
 		doc = soap_xmlParseFile((char*)location TSRMLS_CC);
+		sdl_restore_uri_credentials(ctx TSRMLS_CC);
+
 		if (doc == NULL) {
 			soap_error1(E_ERROR, "Parsing Schema: can't import schema from '%s'", location);
 		}
@@ -453,7 +456,14 @@ static int schema_list(sdlPtr sdl, xmlAttrPtr tns, xmlNodePtr listType, sdlTypeP
 		newType = emalloc(sizeof(sdlType));
 		memset(newType, 0, sizeof(sdlType));
 
-		newType->name = estrdup("anonymous");
+		{
+			smart_str anonymous = {0};
+			
+			smart_str_appendl(&anonymous, "anonymous", sizeof("anonymous")-1);
+			smart_str_append_long(&anonymous, zend_hash_num_elements(sdl->types));
+			smart_str_0(&anonymous);
+			newType->name = anonymous.c;
+		}
 		newType->namens = estrdup((char*)tns->children->content);
 
 		if (cur_type->elements == NULL) {
@@ -463,6 +473,7 @@ static int schema_list(sdlPtr sdl, xmlAttrPtr tns, xmlNodePtr listType, sdlTypeP
 		zend_hash_next_index_insert(cur_type->elements, &newType, sizeof(sdlTypePtr), (void **)&tmp);
 
 		schema_simpleType(sdl, tns, trav, newType);
+
 		trav = trav->next;
 	}
 	if (trav != NULL) {
@@ -541,7 +552,14 @@ static int schema_union(sdlPtr sdl, xmlAttrPtr tns, xmlNodePtr unionType, sdlTyp
 			newType = emalloc(sizeof(sdlType));
 			memset(newType, 0, sizeof(sdlType));
 
-			newType->name = estrdup("anonymous");
+			{
+				smart_str anonymous = {0};
+			
+				smart_str_appendl(&anonymous, "anonymous", sizeof("anonymous")-1);
+				smart_str_append_long(&anonymous, zend_hash_num_elements(sdl->types));
+				smart_str_0(&anonymous);
+				newType->name = anonymous.c;
+			}
 			newType->namens = estrdup((char*)tns->children->content);
 
 			if (cur_type->elements == NULL) {
@@ -1063,8 +1081,8 @@ static int schema_group(sdlPtr sdl, xmlAttrPtr tns, xmlNodePtr groupType, sdlTyp
 			nsptr = xmlSearchNs(groupType->doc, groupType, BAD_CAST(ns));
 			if (nsptr != NULL) {
 				smart_str_appends(&key, (char*)nsptr->href);
-				smart_str_appendc(&key, ':');
 			}
+			smart_str_appendc(&key, ':');
 			smart_str_appends(&key, type);
 			smart_str_0(&key);
 
@@ -1490,9 +1508,9 @@ static int schema_element(sdlPtr sdl, xmlAttrPtr tns, xmlNodePtr element, sdlTyp
 			nsptr = xmlSearchNs(element->doc, element, BAD_CAST(ns));
 			if (nsptr != NULL) {
 				smart_str_appends(&nscat, (char*)nsptr->href);
-				smart_str_appendc(&nscat, ':');
 				newType->namens = estrdup((char*)nsptr->href);
-			}
+			} 
+			smart_str_appendc(&nscat, ':');
 			smart_str_appends(&nscat, type);
 			newType->name = estrdup(type);
 			smart_str_0(&nscat);
@@ -1716,9 +1734,9 @@ static int schema_attribute(sdlPtr sdl, xmlAttrPtr tns, xmlNodePtr attrType, sdl
 			nsptr = xmlSearchNs(attrType->doc, attrType, BAD_CAST(ns));
 			if (nsptr != NULL) {
 				smart_str_appends(&key, (char*)nsptr->href);
-				smart_str_appendc(&key, ':');
 				newAttr->namens = estrdup((char*)nsptr->href);
 			}
+			smart_str_appendc(&key, ':');
 			smart_str_appends(&key, attr_name);
 			smart_str_0(&key);
 			newAttr->ref = estrdup(key.c);
@@ -1879,7 +1897,14 @@ static int schema_attribute(sdlPtr sdl, xmlAttrPtr tns, xmlNodePtr attrType, sdl
 			}
 			dummy_type = emalloc(sizeof(sdlType));
 			memset(dummy_type, 0, sizeof(sdlType));
-			dummy_type->name = estrdup("anonymous");
+			{
+				smart_str anonymous = {0};
+			
+				smart_str_appendl(&anonymous, "anonymous", sizeof("anonymous")-1);
+				smart_str_append_long(&anonymous, zend_hash_num_elements(sdl->types));
+				smart_str_0(&anonymous);
+				dummy_type->name = anonymous.c;
+			}
 			dummy_type->namens = estrdup((char*)tns->children->content);
 			schema_simpleType(sdl, tns, trav, dummy_type);
 			newAttr->encode = dummy_type->encode;
@@ -1945,8 +1970,8 @@ static int schema_attributeGroup(sdlPtr sdl, xmlAttrPtr tns, xmlNodePtr attrGrou
 			nsptr = xmlSearchNs(attrGroup->doc, attrGroup, BAD_CAST(ns));
 			if (nsptr != NULL) {
 				smart_str_appends(&key, (char*)nsptr->href);
-				smart_str_appendc(&key, ':');
 			}
+			smart_str_appendc(&key, ':');
 			smart_str_appends(&key, group_name);
 			smart_str_0(&key);
 			newAttr->ref = estrdup(key.c);
