@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: php_spl.c 305335 2010-11-14 18:40:08Z felipe $ */
+/* $Id: php_spl.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -406,6 +406,7 @@ PHP_FUNCTION(spl_autoload_call)
 			zend_exception_save(TSRMLS_C);
 			if (retval) {
 				zval_ptr_dtor(&retval);
+				retval = NULL;
 			}
 			if (zend_hash_exists(EG(class_table), lc_name, class_name_len + 1)) {
 				break;
@@ -877,7 +878,7 @@ const zend_function_entry spl_functions[] = {
 	PHP_FE(iterator_count,          arginfo_iterator)
 	PHP_FE(iterator_apply,          arginfo_iterator_apply)
 #endif /* SPL_ITERATORS_H */
-	{NULL, NULL, NULL}
+	PHP_FE_END
 };
 /* }}} */
 
