@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: spl_array.c 305335 2010-11-14 18:40:08Z felipe $ */
+/* $Id: spl_array.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -77,6 +77,7 @@ typedef struct _spl_array_object {
 	php_serialize_data_t   *serialize_data;
 	php_unserialize_data_t *unserialize_data;
 	HashTable              *debug_info;
+	unsigned char		   nApplyCount;
 } spl_array_object;
 
 static inline HashTable *spl_array_get_hash_table(spl_array_object* intern, int check_std_props TSRMLS_DC) { /* {{{ */
@@ -579,8 +580,15 @@ static int spl_array_has_dimension_ex(int check_inherited, zval *object, zval *o
 	switch(Z_TYPE_P(offset)) {
 	case IS_STRING:
 		if (check_empty) {
-			if (zend_symtable_find(spl_array_get_hash_table(intern, 0 TSRMLS_CC), Z_STRVAL_P(offset), Z_STRLEN_P(offset)+1, (void **) &tmp) != FAILURE && zend_is_true(*tmp)) {
-				return 1;
+			if (zend_symtable_find(spl_array_get_hash_table(intern, 0 TSRMLS_CC), Z_STRVAL_P(offset), Z_STRLEN_P(offset)+1, (void **) &tmp) != FAILURE) {
+				switch (check_empty) {
+					case 0:
+						return Z_TYPE_PP(tmp) != IS_NULL;
+					case 2:
+						return 1;
+					default:
+						return zend_is_true(*tmp);
+				}
 			}
 			return 0;
 		} else {
@@ -597,8 +605,15 @@ static int spl_array_has_dimension_ex(int check_inherited, zval *object, zval *o
 		}
 		if (check_empty) {
 			HashTable *ht = spl_array_get_hash_table(intern, 0 TSRMLS_CC);
-			if (zend_hash_index_find(ht, index, (void **)&tmp) != FAILURE && zend_is_true(*tmp)) {
-				return 1;
+			if (zend_hash_index_find(ht, index, (void **)&tmp) != FAILURE) {
+				switch (check_empty) {
+					case 0:
+						return Z_TYPE_PP(tmp) != IS_NULL;
+					case 2:
+						return 1;
+					default:
+						return zend_is_true(*tmp);
+				}
 			}
 			return 0;
 		} else {
@@ -714,8 +729,16 @@ SPL_METHOD(Array, getArrayCopy)
 static HashTable *spl_array_get_properties(zval *object TSRMLS_DC) /* {{{ */
 {
 	spl_array_object *intern = (spl_array_object*)zend_object_store_get_object(object TSRMLS_CC);
+	HashTable *result;
 
-	return spl_array_get_hash_table(intern, 1 TSRMLS_CC);
+	if (intern->nApplyCount > 1) {
+		php_error_docref(NULL TSRMLS_CC, E_ERROR, "Nesting level too deep - recursive dependency?");
+	}
+
+	intern->nApplyCount++;
+	result = spl_array_get_hash_table(intern, 1 TSRMLS_CC);
+	intern->nApplyCount--;
+	return result;
 } /* }}} */
 
 static HashTable* spl_array_get_debug_info(zval *obj, int *is_temp TSRMLS_DC) /* {{{ */
@@ -1915,7 +1938,7 @@ static const zend_function_entry spl_funcs_ArrayObject[] = {
 	SPL_ME(Array, exchangeArray,    arginfo_array_exchangeArray,    ZEND_ACC_PUBLIC)
 	SPL_ME(Array, setIteratorClass, arginfo_array_setIteratorClass, ZEND_ACC_PUBLIC)
 	SPL_ME(Array, getIteratorClass, arginfo_array_void,             ZEND_ACC_PUBLIC)
-	{NULL, NULL, NULL}
+	PHP_FE_END
 };
 
 static const zend_function_entry spl_funcs_ArrayIterator[] = {
@@ -1944,13 +1967,13 @@ static const zend_function_entry spl_funcs_ArrayIterator[] = {
 	SPL_ME(Array, next,             arginfo_array_void,             ZEND_ACC_PUBLIC)
 	SPL_ME(Array, valid,            arginfo_array_void,             ZEND_ACC_PUBLIC)
 	SPL_ME(Array, seek,             arginfo_array_seek,             ZEND_ACC_PUBLIC)
-	{NULL, NULL, NULL}
+	PHP_FE_END
 };
 
 static const zend_function_entry spl_funcs_RecursiveArrayIterator[] = {
 	SPL_ME(Array, hasChildren,   arginfo_array_void, ZEND_ACC_PUBLIC)
 	SPL_ME(Array, getChildren,   arginfo_array_void, ZEND_ACC_PUBLIC)
-	{NULL, NULL, NULL}
+	PHP_FE_END
 };
 /* }}} */
 
