@@ -1,8 +1,8 @@
-/* 
+/*
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2008 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,7 +18,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: datetime.c,v 1.134.2.2.2.5 2007/12/31 07:20:12 sebastian Exp $ */
+/* $Id: datetime.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #include "php.h"
 #include "zend_operators.h"
@@ -49,7 +49,7 @@ char *day_short_names[] = {
 	"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
 };
 
-/* {{{ php_std_date
+/* {{{ PHPAPI char *php_std_date(time_t t TSRMLS_DC)
    Return date string in standard format for http headers */
 PHPAPI char *php_std_date(time_t t TSRMLS_DC)
 {
@@ -79,13 +79,12 @@ PHPAPI char *php_std_date(time_t t TSRMLS_DC)
 				((tm1->tm_year) % 100),
 				tm1->tm_hour, tm1->tm_min, tm1->tm_sec);
 	}
-	
+
 	str[79] = 0;
 	return (str);
 }
 /* }}} */
 
-
 #if HAVE_STRPTIME
 #ifndef HAVE_STRPTIME_DECL_FAILS
 char *strptime(const char *s, const char *format, struct tm *tm);
@@ -102,8 +101,7 @@ PHP_FUNCTION(strptime)
 	struct tm  parsed_time;
 	char      *unparsed_part;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss", 
-		&ts, &ts_length, &format, &format_length) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss", &ts, &ts_length, &format, &format_length) == FAILURE) {
 		return;
 	}
 
@@ -126,6 +124,7 @@ PHP_FUNCTION(strptime)
 	add_assoc_string(return_value, "unparsed", unparsed_part, 1);
 }
 /* }}} */
+
 #endif
 
 /*
