@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,7 +18,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: dns.c 304301 2010-10-11 03:07:03Z cataphract $ */
+/* $Id: dns.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 /* {{{ includes */
 #include "php.h"
@@ -62,6 +62,16 @@
 #define AF_INET 2   /* internetwork: UDP, TCP, etc. */
 #endif
 
+#ifndef MAXHOSTNAMELEN
+#define MAXHOSTNAMELEN 255
+#endif
+
+/* For the local hostname obtained via gethostname which is different from the
+   dns-related MAXHOSTNAMELEN constant above */
+#ifndef HOST_NAME_MAX
+#define HOST_NAME_MAX 255
+#endif
+
 #include "php_dns.h"
 
 /* type compat */
@@ -118,7 +128,7 @@ static char *php_gethostbyname(char *name);
    Get the host name of the current machine */
 PHP_FUNCTION(gethostname)
 {
-	char buf[4096];
+	char buf[HOST_NAME_MAX];
 
 	if (zend_parse_parameters_none() == FAILURE) {
 		return;
@@ -794,12 +804,14 @@ PHP_FUNCTION(dns_get_record)
 #if defined(HAVE_DNS_SEARCH)
 			handle = dns_open(NULL);
 			if (handle == NULL) {
+				zval_dtor(return_value);
 				RETURN_FALSE;
 			}
 #elif defined(HAVE_RES_NSEARCH)
 		    memset(&state, 0, sizeof(state));
 		    if (res_ninit(handle)) {
-					RETURN_FALSE;
+		    	zval_dtor(return_value);
+				RETURN_FALSE;
 			}
 #else
 			res_init();
