@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2011 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -13,13 +13,13 @@
    | license@php.net so we can mail you a copy immediately.               |
    +----------------------------------------------------------------------+
    | Authors: Jim Winstead <jimw@php.net>                                 |
-   |          Stig Sther Bakken <ssb@php.net>                            |
+   |          Stig Sæther Bakken <ssb@php.net>                            |
    |          Zeev Suraski <zeev@zend.com>                                |
    | PHP 4.0 patches by Thies C. Arntzen <thies@thieso.net>               |
    +----------------------------------------------------------------------+
 */
 
-/* $Id: math.c 306939 2011-01-01 02:19:59Z felipe $ */
+/* $Id: math.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #include "php.h"
 #include "php_math.h"
@@ -92,6 +92,18 @@ static inline double php_intpow10(int power) {
 }
 /* }}} */
 
+/* {{{ php_math_is_finite */
+static inline int php_math_is_finite(double value) {
+#if defined(PHP_WIN32)
+	return _finite(value);
+#elif defined(isfinite)
+	return isfinite(value);
+#else
+	return value == value && (value == 0. || value * 2. != value);
+#endif
+}
+/* }}} */
+
 /* {{{ php_round_helper
        Actually performs the rounding of a value to integer in a certain mode */
 static inline double php_round_helper(double value, int mode) {
@@ -129,11 +141,11 @@ PHPAPI double _php_math_round(double value, int places, int mode) {
 	double tmp_value;
 	int precision_places;
 
-	if ((precision_places = php_intlog10abs(value)) > 0) {
-		precision_places = 14 - php_intlog10abs(value);
-	} else {
-		precision_places = 14;
+	if (!php_math_is_finite(value)) {
+		return value;
 	}
+	
+	precision_places = 14 - php_intlog10abs(value);
 
 	f1 = php_intpow10(abs(places));
 
