@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2010 The PHP Group                                |
+  | Copyright (c) 1997-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: streamsfuncs.c 304384 2010-10-14 03:15:15Z cataphract $ */
+/* $Id: streamsfuncs.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #include "php.h"
 #include "php_globals.h"
@@ -413,29 +413,36 @@ PHP_FUNCTION(stream_socket_recvfrom)
    Reads all remaining bytes (or up to maxlen bytes) from a stream and returns them as a string. */
 PHP_FUNCTION(stream_get_contents)
 {
-	php_stream *stream;
-	zval *zsrc;
-	long maxlen = PHP_STREAM_COPY_ALL, pos = -1L;
-	int len, newlen;
-	char *contents = NULL;
+	php_stream	*stream;
+	zval		*zsrc;
+	long		maxlen		= PHP_STREAM_COPY_ALL,
+				desiredpos	= -1L;
+	int			len,
+				newlen;
+	char		*contents	= NULL;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|ll", &zsrc, &maxlen, &pos) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|ll", &zsrc, &maxlen, &desiredpos) == FAILURE) {
 		RETURN_FALSE;
 	}
 
 	php_stream_from_zval(stream, &zsrc);
 
-	if (pos >= 0) {
-		int seek_res = 0;
-		if (pos > stream->position) {
+	if (desiredpos >= 0) {
+		int		seek_res = 0;
+		off_t	position;
+
+		position = php_stream_tell(stream);
+		if (position >= 0 && desiredpos > position) {
 			/* use SEEK_CUR to allow emulation in streams that don't support seeking */
-			seek_res = php_stream_seek(stream, pos - stream->position, SEEK_CUR);
-		} else if (pos < stream->position)  {
-			seek_res = php_stream_seek(stream, pos, SEEK_SET);
+			seek_res = php_stream_seek(stream, desiredpos - position, SEEK_CUR);
+		} else if (desiredpos < position)  {
+			/* desired position before position or error on tell */
+			seek_res = php_stream_seek(stream, desiredpos, SEEK_SET);
 		}
 
 		if (seek_res != 0) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Failed to seek to position %ld in the stream", pos);
+			php_error_docref(NULL TSRMLS_CC, E_WARNING,
+				"Failed to seek to position %ld in the stream", desiredpos);
 			RETURN_FALSE;
 		}
 	}
@@ -627,7 +634,7 @@ static int stream_array_to_fd_set(zval *stream_array, fd_set *fds, php_socket_t
 		 * when casting.  It is only used here so that the buffered data warning
 		 * is not displayed.
 		 * */
-		if (SUCCESS == php_stream_cast(stream, PHP_STREAM_AS_FD_FOR_SELECT | PHP_STREAM_CAST_INTERNAL, (void*)&this_fd, 1) && this_fd >= 0) {
+		if (SUCCESS == php_stream_cast(stream, PHP_STREAM_AS_FD_FOR_SELECT | PHP_STREAM_CAST_INTERNAL, (void*)&this_fd, 1) && this_fd != -1) {
 
 			PHP_SAFE_FD_SET(this_fd, fds);
 
@@ -667,7 +674,7 @@ static int stream_array_from_fd_set(zval *stream_array, fd_set *fds TSRMLS_DC)
 		 * when casting.  It is only used here so that the buffered data warning
 		 * is not displayed.
 		 */
-		if (SUCCESS == php_stream_cast(stream, PHP_STREAM_AS_FD_FOR_SELECT | PHP_STREAM_CAST_INTERNAL, (void*)&this_fd, 1) && this_fd >= 0) {
+		if (SUCCESS == php_stream_cast(stream, PHP_STREAM_AS_FD_FOR_SELECT | PHP_STREAM_CAST_INTERNAL, (void*)&this_fd, 1) && this_fd != -1) {
 			if (PHP_SAFE_FD_ISSET(this_fd, fds)) {
 				zend_hash_next_index_insert(new_hash, (void *)elem, sizeof(zval *), (void **)&dest_elem);
 				if (dest_elem) {
