@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2010 The PHP Group                                |
+  | Copyright (c) 1997-2012 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: tidy.c 305507 2010-11-18 15:22:22Z pajoye $ */
+/* $Id: tidy.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -404,7 +404,7 @@ static const zend_function_entry tidy_functions[] = {
 	PHP_FE(tidy_get_html,		arginfo_tidy_get_html)
 	PHP_FE(tidy_get_body,		arginfo_tidy_get_body)
 	PHP_FE(ob_tidyhandler,		arginfo_ob_tidyhandler)
-	{NULL, NULL, NULL}
+	PHP_FE_END
 };
 
 static const zend_function_entry tidy_funcs_doc[] = {
@@ -429,7 +429,7 @@ static const zend_function_entry tidy_funcs_doc[] = {
 	TIDY_METHOD_MAP(html, tidy_get_html, NULL)
 	TIDY_METHOD_MAP(body, tidy_get_body, NULL)
 	TIDY_DOC_ME(__construct, NULL)
-	{NULL, NULL, NULL}
+	PHP_FE_END
 };
 
 static const zend_function_entry tidy_funcs_node[] = {
@@ -443,7 +443,7 @@ static const zend_function_entry tidy_funcs_node[] = {
 	TIDY_NODE_ME(isPhp, NULL)
 	TIDY_NODE_ME(getParent, NULL)
 	TIDY_NODE_PRIVATE_ME(__construct, NULL)
-	{NULL, NULL, NULL}
+	PHP_FE_END
 };
 
 static zend_class_entry *tidy_ce_doc, *tidy_ce_node;
@@ -985,6 +985,10 @@ static void php_tidy_create_node(INTERNAL_FUNCTION_PARAMETERS, tidy_base_nodetyp
 		case is_body_node:
 			node = tidyGetBody(obj->ptdoc->doc);
 			break;
+
+		default:
+			RETURN_NULL();
+			break;
 	}
 
 	if (!node) {
@@ -1088,7 +1092,7 @@ static PHP_MINFO_FUNCTION(tidy)
 	php_info_print_table_start();
 	php_info_print_table_header(2, "Tidy support", "enabled");
 	php_info_print_table_row(2, "libTidy Release", (char *)tidyReleaseDate());
-	php_info_print_table_row(2, "Extension Version", PHP_TIDY_MODULE_VERSION " ($Id: tidy.c 305507 2010-11-18 15:22:22Z pajoye $)");
+	php_info_print_table_row(2, "Extension Version", PHP_TIDY_MODULE_VERSION " ($Id: tidy.c 321634 2012-01-01 13:15:04Z felipe $)");
 	php_info_print_table_end();
 
 	DISPLAY_INI_ENTRIES();
@@ -1284,7 +1288,7 @@ static PHP_FUNCTION(tidy_diagnose)
 {
 	TIDY_FETCH_OBJECT;
 
-	if (tidyRunDiagnostics(obj->ptdoc->doc) >= 0) {
+	if (tidyStatus(obj->ptdoc->doc) != 0 && tidyRunDiagnostics(obj->ptdoc->doc) >= 0) {
 		tidy_doc_update_properties(obj TSRMLS_CC);
 		RETURN_TRUE;
 	}
