@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: reentrancy.c 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id: reentrancy.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #include <sys/types.h>
 #include <string.h>
@@ -60,14 +60,14 @@ static MUTEX_T reentrant_locks[NUMBER_OF_LOCKS];
 
 PHPAPI char *php_ctime_r(const time_t *clock, char *buf)
 {
-	if (ctime_r(clock, buf, 26) == buf)
+	if (ctime_r(clock, buf) == buf)
 		return (buf);
 	return (NULL);
 }
 
 PHPAPI char *php_asctime_r(const struct tm *tm, char *buf)
 {
-	if (asctime_r(tm, buf, 26) == buf)
+	if (asctime_r(tm, buf) == buf)
 		return (buf);
 	return (NULL);
 }
