@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2010 The PHP Group                                |
+   | Copyright (c) 1997-2012 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -21,7 +21,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: cgi_main.c 293036 2010-01-03 09:23:27Z sebastian $ */
+/* $Id: cgi_main.c 321634 2012-01-01 13:15:04Z felipe $ */
 
 #include "php.h"
 #include "php_globals.h"
@@ -725,7 +725,7 @@ static void php_cgi_ini_activate_user_config(char *path, int path_len, const cha
 
 	/* Check whether cache entry has expired and rescan if it is */
 	if (request_time > entry->expires) {
-		char * real_path;
+		char *real_path = NULL;
 		int real_path_len;
 		char *s1, *s2;
 		int s_len;
@@ -735,6 +735,10 @@ static void php_cgi_ini_activate_user_config(char *path, int path_len, const cha
 
 		if (!IS_ABSOLUTE_PATH(path, path_len)) {
 			real_path = tsrm_realpath(path, NULL TSRMLS_CC);
+			/* see #51688, looks like we may get invalid path as doc root using cgi with apache */
+			if (real_path == NULL) {
+				return;
+			}
 			real_path_len = strlen(real_path);
 			path = real_path;
 			path_len = real_path_len;
@@ -770,6 +774,9 @@ static void php_cgi_ini_activate_user_config(char *path, int path_len, const cha
 			php_parse_user_ini_file(path, PG(user_ini_filename), entry->user_config TSRMLS_CC);
 		}
 
+		if (real_path) {
+			free(real_path);
+		}
 		entry->expires = request_time + PG(user_ini_cache_ttl);
 	}
 
@@ -1346,7 +1353,7 @@ static void init_request_info(TSRMLS_D)
 		/* FIXME - Work out proto_num here */
 		SG(request_info).query_string = sapi_cgibin_getenv("QUERY_STRING", sizeof("QUERY_STRING")-1 TSRMLS_CC);
 		SG(request_info).content_type = (content_type ? content_type : "" );
-		SG(request_info).content_length = (content_length ? atoi(content_length) : 0);
+		SG(request_info).content_length = (content_length ? atol(content_length) : 0);
 
 		/* The CGI RFC allows servers to pass on unvalidated Authorization data */
 		auth = sapi_cgibin_getenv("HTTP_AUTHORIZATION", sizeof("HTTP_AUTHORIZATION")-1 TSRMLS_CC);
@@ -1875,6 +1882,7 @@ consult the installation file that came with this distribution, or visit \n\
 							}
 							php_print_info(0xFFFFFFFF TSRMLS_CC);
 							php_request_shutdown((void *) 0);
+							fcgi_shutdown();
 							exit_status = 0;
 							goto out;
 
@@ -1896,6 +1904,7 @@ consult the installation file that came with this distribution, or visit \n\
 							print_extensions(TSRMLS_C);
 							php_printf("\n");
 							php_end_ob_buffers(1 TSRMLS_CC);
+							fcgi_shutdown();
 							exit_status = 0;
 							goto out;
 
@@ -1924,11 +1933,12 @@ consult the installation file that came with this distribution, or visit \n\
 								SG(request_info).no_headers = 1;
 							}
 #if ZEND_DEBUG
-							php_printf("PHP %s (%s) (built: %s %s) (DEBUG)\nCopyright (c) 1997-2010 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__, __TIME__, get_zend_version());
+							php_printf("PHP %s (%s) (built: %s %s) (DEBUG)\nCopyright (c) 1997-2012 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__, __TIME__, get_zend_version());
 #else
-							php_printf("PHP %s (%s) (built: %s %s)\nCopyright (c) 1997-2010 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__, __TIME__, get_zend_version());
+							php_printf("PHP %s (%s) (built: %s %s)\nCopyright (c) 1997-2012 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__, __TIME__, get_zend_version());
 #endif
 							php_request_shutdown((void *) 0);
+							fcgi_shutdown();
 							exit_status = 0;
 							goto out;
 
